package com.muyu.etl;

import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.pool.DruidPooledConnection;
import com.muyu.cache.AssetCache;
import com.muyu.common.core.constant.SecurityConstants;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.redis.service.RedisService;
import com.muyu.etl.domain.BasicConfigInfo;
import com.muyu.etl.util.factory.DruidUtilsFactory;
import com.muyu.etl.util.service.ConnectionPoolFactory;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;

import javax.servlet.ServletException;
import java.sql.SQLException;
import java.util.List;

/**
 * @ClassName AssetClientRunner
 * @Description 描述
 * @Author SaiSai.Liu
 * @Date 2024/5/9 21:26
 */
@Component
@Log4j2
public class AssetClientRunner implements ApplicationRunner {

    @Autowired
    private ConnectionPoolFactory connectionPoolFactory;

    @Autowired
    private RedisService redisService;

    @Override
    public void run(ApplicationArguments args) throws ServletException {
        AssetCache assetCache = new AssetCache(redisService);
        List<List<BasicConfigInfo>> basicConfigInfos = assetCache.get(assetCache.preKey()+"basic");
        System.out.println(basicConfigInfos.get(0));
        if (basicConfigInfos.get(0).isEmpty()) throw new ServletException("初始化调用失败，无数据");
        for (BasicConfigInfo basicConfigInfo : basicConfigInfos.get(0)) {
            try {
                log.info("初始化开始");
                connectionPoolFactory.init(basicConfigInfo);
            } catch (Exception e) {
                log.error("初始化异常：{}", e.getMessage());
                //跳过
                continue;
            }
        }
        log.info("初始化完成");
    }


}
