
package com.muyu.etl.util.factory;

import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.pool.DruidPooledConnection;
import com.muyu.etl.domain.BasicConfigInfo;
import com.muyu.etl.util.service.ConnectionPoolFactory;
import lombok.extern.log4j.Log4j2;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;

@Log4j2
public class DruidUtilsFactory implements ConnectionPoolFactory<BasicConfigInfo> {

    public static final ThreadLocal<Connection> threadConn = new ThreadLocal<>();
    //测试用
    public static final ThreadLocal<DruidDataSource> threadSource = new ThreadLocal<>();
    // 连接池容器
    public static HashMap<String, DruidDataSource> map = new HashMap<>();

    /**
     * 初始化连接池
     *
     * @param basicConfigInfo
     * @return
     * @throws SQLException
     */
    @Override
    public DruidDataSource init(BasicConfigInfo basicConfigInfo) throws SQLException {
        // 连接池对象
        DruidDataSource source = new DruidDataSource();
        //  定义下面需要的对象
        String host = basicConfigInfo.getHost();
        String port = basicConfigInfo.getPort();
        String databaseName = basicConfigInfo.getDatabaseName();
        String databaseType = basicConfigInfo.getDatabaseType();
        // 引擎配置
        source.setUrl("jdbc:" + databaseType + "://" + host + ":" + port + "/" + databaseName + "?" + basicConfigInfo.getConnectionParams());
        source.setUsername(basicConfigInfo.getUsername());
        source.setPassword(basicConfigInfo.getPassword());
        //初始化连接量
        source.setInitialSize(Math.toIntExact(basicConfigInfo.getInitLinkNum()));
        //最大连接数
        source.setMaxActive(Math.toIntExact(basicConfigInfo.getMaxLinkNum()));
        source.setMaxWaitThreadCount(Math.toIntExact(basicConfigInfo.getMaxWaitTimes()));
        //连接池中的最大最小生存时间
        source.setMaxEvictableIdleTimeMillis(basicConfigInfo.getMaxWaitTime());
        source.setMinEvictableIdleTimeMillis(30000);
        //最大等待时间
        source.setTimeBetweenEvictionRunsMillis(30000);
        //进程最大等待时间
        source.setMaxWait(basicConfigInfo.getMaxWaitTime());
        map.put(host + ":" + port + "/" + databaseName, source);
        //  获取并返回连接池对象
        return source;
    }

    /**
     * 根据接入取出一个连接对象
     *
     * @param basicConfigInfo
     * @return
     */
    @Override
    public Connection getConnection(BasicConfigInfo basicConfigInfo)  {
        String host = basicConfigInfo.getHost();
        String port = basicConfigInfo.getPort();
        String databaseName = basicConfigInfo.getDatabaseName();
        DruidDataSource druidDataSource = map.get(host + ":" + port + "/" + databaseName);
        threadSource.set(druidDataSource);
        DruidDataSource init = null;
        try {
            init = druidDataSource;
            if (init==null){
                init = this.init(basicConfigInfo);
            }
            return init.getConnection();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 将连接对象放回连接池
     * @throws SQLException
     */
    @Override
    public void giveBack() throws SQLException {
        DruidDataSource source = threadSource.get();
        //当前活跃连接数
        int activeCount = source.getActiveCount();
        log.info("当前活跃连接数：{}",activeCount);
        Connection connection = threadConn.get();
        connection.close();
        log.info("连接已归还");
        threadConn.remove();
        activeCount = source.getActiveCount();
        log.info("关闭后当前活跃连接数：{}",activeCount);
    }


}
