package com.muyu.service.impl;

import com.alibaba.druid.pool.DruidDataSource;
import com.muyu.etl.domain.BasicConfigInfo;
import com.muyu.etl.domain.TableInfo;
import com.muyu.etl.util.factory.DruidUtilsFactory;
import com.muyu.etl.util.service.ConnectionPoolFactory;
import com.muyu.service.UnitService;
import com.mysql.cj.xdevapi.Table;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.ServletException;
import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * @ClassName UnitServiceImpl
 * @Description 描述
 * @Author SaiSai.Liu
 * @Date 2024/5/15 22:10
 */
@Log4j2
@Service
public class UnitServiceImpl implements UnitService {

    @Autowired
    private DruidUtilsFactory druidUtilsFactory;
    private HashMap<String, BasicConfigInfo> basicConfigInfoHashMap = new HashMap<>();
    @Override
    public List<TableInfo> getTableInfoList(BasicConfigInfo basicConfigInfo)  {
        List<TableInfo> tableInfos = new ArrayList<>();
        Connection connection = druidUtilsFactory.getConnection(basicConfigInfo);
        DruidUtilsFactory.threadConn.set(connection);
        if (connection == null) {
            try {
                throw new ServletException("获取连接失败");
            } catch (ServletException e) {
                throw new RuntimeException(e);
            }
        }
        try {
            DatabaseMetaData metaData = connection.getMetaData();
            ResultSet rs = metaData.getTables(basicConfigInfo.getDatabaseName(), null,
                    "%", new String[]{"TABLE", "VIEW"});
            while (rs.next()){
                DruidDataSource source = DruidUtilsFactory.threadSource.get();
                int activeCount = source.getActiveCount();
                log.info("当前连接数：{}",activeCount);
                String tableName = rs.getString("TABLE_NAME");
                String remark = rs.getString("REMARKS");
                PreparedStatement ps = connection.prepareStatement("Select COUNT(*) FROM " + tableName);
                ResultSet rset = ps.executeQuery();
                Long rowCount = 0L;
                while (rset.next()){
                    rowCount = rset.getLong(1);
                }
                TableInfo tableInfo = TableInfo.builder()
                        .tableRemark(remark)
                        .tableName(tableName)
                        .basicId(basicConfigInfo.getId())
                        .dataNum(rowCount)
                        .build();
                tableInfos.add(tableInfo);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }finally {
            try {
                druidUtilsFactory.giveBack();

            } catch (SQLException e) {
                throw new RuntimeException("连接池归还失败");
            }
        }
        return tableInfos;
    }
}
